//+--------------------------------------------------------------------------------------------+
//|                                                                               MarkBars.mq4 |
//|                                                             Copyright  2012, Serega Lykov |
//|                                                                 http://mtexperts.narod.ru/ |
//+--------------------------------------------------------------------------------------------+

#property copyright "Copyright  2012, Serega Lykov"
#property link      "http://mtexperts.narod.ru/"

//---- property of indicator ------------------------------------------------------------------+
#property indicator_chart_window
#property indicator_buffers 6
#property indicator_color1 Blue
#property indicator_color2 Blue
#property indicator_color3 Red
#property indicator_color4 Red
#property indicator_color5 Gold
#property indicator_color6 Gold

//---- constants ------------------------------------------------------------------------------+
#define ObjectsSignature                "MarkBars_"

//---- external parameters --------------------------------------------------------------------+
extern int    NumBars                         = 1;
extern int    TakeProfit                      = 50;
extern int    StopLoss                        = 100;
extern int    Spread                          = 4;
extern bool   DrawRectangles                  = false;
extern color  DrawRectangles_ColorBuy         = Blue;
extern color  DrawRectangles_ColorSell        = Red;
extern color  DrawRectangles_ColorBoth        = Gold;

//---- buffers --------------------------------------------------------------------------------+
static double BuyBar_H[];
static double BuyBar_L[];
static double SellBar_H[];
static double SellBar_L[];
static double BothBar_H[];
static double BothBar_L[];

//---- global variables -----------------------------------------------------------------------+
static int    period_seconds;
static double point;
static double spread;
static double main_tp;
static double main_sl;
static string symbol;
static bool   use_tp;
static bool   use_sl;
static bool   use_levels;

//---------------------------------------------------------------------------------------------+
//---- init -----------------------------------------------------------------------------------+
//---------------------------------------------------------------------------------------------+
int init()
  {
   //---- initialization values of constants --------------------------------------------------+
   period_seconds = Period() * 60;
   if(Digits < 4) point = 0.01;
   else point = 0.0001;
   if(NumBars <= 0) NumBars = 1;
   spread  = NormalizeDouble(Spread * point,Digits);
   main_sl = NormalizeDouble(StopLoss * point,Digits);
   main_tp = NormalizeDouble(TakeProfit * point,Digits);
   symbol = Symbol();
   use_sl = StopLoss > 0.0;
   use_tp = TakeProfit > 0.0;
   use_levels = use_sl || use_tp;
   //---- set a "short" name of the indicator -------------------------------------------------+
   IndicatorShortName("MarkBars");
   //---- the 0 additional buffer -------------------------------------------------------------+
   IndicatorBuffers(6);
   //---- set an accuracy of values of the indicator ------------------------------------------+
   IndicatorDigits(Digits);
   //---- set a style for line ----------------------------------------------------------------+
   SetIndexStyle(0,DRAW_HISTOGRAM,STYLE_SOLID,4);
   SetIndexStyle(1,DRAW_HISTOGRAM,STYLE_SOLID,4);
   SetIndexStyle(2,DRAW_HISTOGRAM,STYLE_SOLID,4);
   SetIndexStyle(3,DRAW_HISTOGRAM,STYLE_SOLID,4);
   SetIndexStyle(4,DRAW_HISTOGRAM,STYLE_SOLID,4);
   SetIndexStyle(5,DRAW_HISTOGRAM,STYLE_SOLID,4);
   //---- set an arrays for line --------------------------------------------------------------+
   SetIndexBuffer(0,BuyBar_H);
   SetIndexBuffer(1,BuyBar_L);
   SetIndexBuffer(2,SellBar_H);
   SetIndexBuffer(3,SellBar_L);
   SetIndexBuffer(4,BothBar_H);
   SetIndexBuffer(5,BothBar_L);
   //---- set a names for lines ---------------------------------------------------------------+
   SetIndexLabel(0,"Buy_H");
   SetIndexLabel(1,"Buy_L");
   SetIndexLabel(2,"Sell_H");
   SetIndexLabel(3,"Sell_L");
   SetIndexLabel(4,"Both_H");
   SetIndexLabel(5,"Both_L");
   //---- finish of initialization ------------------------------------------------------------+
   return(0);
   Print("---- Programming by Serega Lykov, http://mtexperts.narod.ru/ ----");
  }

//---------------------------------------------------------------------------------------------+
//---- deinit ---------------------------------------------------------------------------------+
//---------------------------------------------------------------------------------------------+
int deinit()
  {
   if(DrawRectangles) DeleteObjects(ObjectsSignature);
   return(0);
  }

//---------------------------------------------------------------------------------------------+
//---- start ----------------------------------------------------------------------------------+
//---------------------------------------------------------------------------------------------+
int start()
  {
   if(Spread < 0.0) spread = NormalizeDouble(MarketInfo(symbol,MODE_SPREAD) * Point,Digits);
   //---- amount not changed bars after last call of the indicator ----------------------------+
   int counted_bars = IndicatorCounted();
   if(counted_bars < 0) return(-1);
   //---- last counted bar will be counted ----------------------------------------------------+
   if(counted_bars > 0) counted_bars--;
   int limit = Bars - counted_bars;
   if(limit < NumBars) limit = NumBars;
   //---- detect bars -------------------------------------------------------------------------+
   for(int i=limit; i>=0; i--)
     {
      double open  = NormalizeDouble(Open [i],Digits);
      double close = NormalizeDouble(Close[i],Digits);
      double low   = NormalizeDouble(Low  [i],Digits);
      double high  = NormalizeDouble(High [i],Digits);
      if(NumBars > 1)
        {
         int k = NumBars - 1;
         int n = i - 1;
         while(k > 0 && n >= 0)
           {
            close = NormalizeDouble(Close[n],Digits);
            double low_n  = NormalizeDouble(Low [n],Digits);
            double high_n = NormalizeDouble(High[n],Digits);
            if(low  > low_n)  low  = low_n;
            if(high < high_n) high = high_n;
            n--;
            k--;
           }
        }
      if(!use_levels)
        {
         double buy_profit  = NormalizeDouble(close - (open + spread),Digits);
         double sell_profit = NormalizeDouble(open - (close + spread),Digits);
         bool buy_bar  = buy_profit  > 0.0; 
         bool sell_bar = sell_profit > 0.0; 
        }
      else
        {
         double buy_tp  = NormalizeDouble(high - (open + spread),Digits);
         double buy_sl  = NormalizeDouble((open + spread) - low, Digits);
         double sell_tp = NormalizeDouble(open - (low + spread), Digits);
         double sell_sl = NormalizeDouble((high + spread) - open,Digits);
         buy_bar  = (!use_tp || buy_tp  >= main_tp) && (!use_sl || buy_sl  < main_sl);
         sell_bar = (!use_tp || sell_tp >= main_tp) && (!use_sl || sell_sl < main_sl);
        }
      if(!DrawRectangles)
        {
         if(buy_bar)
           {
            if(sell_bar)
              {
               BuyBar_H [i] = EMPTY_VALUE;
               BuyBar_L [i] = EMPTY_VALUE;
               SellBar_H[i] = EMPTY_VALUE;
               SellBar_L[i] = EMPTY_VALUE;
               BothBar_H[i] = high;
               BothBar_L[i] = low;
              }
            else
              {
               BuyBar_H [i] = high;
               BuyBar_L [i] = open;
               SellBar_H[i] = EMPTY_VALUE;
               SellBar_L[i] = EMPTY_VALUE;
               BothBar_H[i] = EMPTY_VALUE;
               BothBar_L[i] = EMPTY_VALUE;
              }
           }
         else
           {
            if(sell_bar)
              {
               BuyBar_H [i] = EMPTY_VALUE;
               BuyBar_L [i] = EMPTY_VALUE;
               SellBar_H[i] = open;
               SellBar_L[i] = low;
               BothBar_H[i] = EMPTY_VALUE;
               BothBar_L[i] = EMPTY_VALUE;
              }
            else
              {
               BuyBar_H [i] = EMPTY_VALUE;
               BuyBar_L [i] = EMPTY_VALUE;
               SellBar_H[i] = EMPTY_VALUE;
               SellBar_L[i] = EMPTY_VALUE;
               BothBar_H[i] = EMPTY_VALUE;
               BothBar_L[i] = EMPTY_VALUE;
              }
           }
        }
      else
        {
         if(buy_bar)
           {
            if(sell_bar)
              {
               if(i == 0) datetime obj_time2 = Time[i] + period_seconds;
               else obj_time2 = Time[i-1];
               CreateObjectRectangle(StringConcatenate(ObjectsSignature,TimeToStr(Time[i],TIME_DATE|TIME_MINUTES|TIME_SECONDS)),Time[i],low,obj_time2,high,DrawRectangles_ColorBoth);
              }
            else
              {
               if(i == 0) obj_time2 = Time[i] + period_seconds;
               else obj_time2 = Time[i-1];
               CreateObjectRectangle(StringConcatenate(ObjectsSignature,TimeToStr(Time[i],TIME_DATE|TIME_MINUTES|TIME_SECONDS)),Time[i],open,obj_time2,high,DrawRectangles_ColorBuy);
              }
           }
         else
           {
            if(sell_bar)
              {
               if(i == 0) obj_time2 = Time[i] + period_seconds;
               else obj_time2 = Time[i-1];
               CreateObjectRectangle(StringConcatenate(ObjectsSignature,TimeToStr(Time[i],TIME_DATE|TIME_MINUTES|TIME_SECONDS)),Time[i],open,obj_time2,low,DrawRectangles_ColorSell);
              }
           }
        }
     }
   //---- finish of iteration -----------------------------------------------------------------+
   return(0);
  }

//---------------------------------------------------------------------------------------------+
//---- DeleteObjects --------------------------------------------------------------------------+
//-------------------------------- programming by Serega Lykov, http://mtexperts.narod.ru/ ----+
//---------------------------------------------------------------------------------------------+
void DeleteObjects(string obj_name_signature, int obj_type=-1)
  {
   int obj_total = ObjectsTotal() - 1;
   for(int i=obj_total; i>=0; i--)
     {
      string obj_name = ObjectName(i);
      if(obj_type >= 0) if(ObjectType(obj_name) != obj_type) continue;
      if(StringFind(obj_name,obj_name_signature,0) != 0) continue;
      ObjectDelete(obj_name);
     }
  }

//---------------------------------------------------------------------------------------------+
//---- CreateObjectRectangle ------------------------------------------------------------------+
//-------------------------------- programming by Serega Lykov, http://mtexperts.narod.ru/ ----+
//---------------------------------------------------------------------------------------------+
void CreateObjectRectangle(string obj_name, datetime obj_time1, double obj_price1, datetime obj_time2, double obj_price2, color obj_color, int obj_window = 0, bool obj_back = true)
  {
   if(ObjectFind(obj_name) < 0) ObjectCreate(obj_name,OBJ_RECTANGLE,obj_window,obj_time1,obj_price1,obj_time2,obj_price2);
   else
     {
      if(ObjectGet(obj_name,OBJPROP_TIME1) != obj_time1 || NormalizeDouble(ObjectGet(obj_name,OBJPROP_PRICE1),Digits) != obj_price1)
        {
         ObjectMove(obj_name,0,obj_time1,obj_price1);
        }
      if(ObjectGet(obj_name,OBJPROP_TIME2) != obj_time2 || NormalizeDouble(ObjectGet(obj_name,OBJPROP_PRICE2),Digits) != obj_price2)
        {
         ObjectMove(obj_name,1,obj_time2,obj_price2);
        }
     }
   ObjectSet(obj_name,OBJPROP_COLOR,obj_color); 
   ObjectSet(obj_name,OBJPROP_BACK, obj_back);
  }

//-------------------------------- programming by Serega Lykov, http://mtexperts.narod.ru/ ----+